@@ -1,7 +1,7 @@
 package App::MysqlUtils;
 
 our $DATE = '2017-06-15'; # DATE
-our $VERSION = '0.005'; # VERSION
+our $VERSION = '0.006'; # VERSION
 
 use 5.010001;
 use strict;
@@ -336,14 +336,34 @@ sub mysql_query {
     [200, "OK", \@rows, {'table.fields'=>\@columns}];
 }
 
-$SPEC{mysql_split_sql_dump_per_table} = {
+$SPEC{mysql_sql_dump_extract_tables} = {
     v => 1.1,
     summary => 'Parse SQL dump and spit out tables to separate files',
     args => {
-        # XXX include_table
-        # XXX include_table_pattern
-        # XXX exclude_table
-        # XXX exclude_table_pattern
+        include_tables => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'include_table',
+            schema => ['array*', of=>'str*'],
+            tags => ['category:filtering'],
+        },
+        exclude_tables => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'exclude_table',
+            schema => ['array*', of=>'str*'],
+            tags => ['category:filtering'],
+        },
+        include_table_patterns => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'include_table_pattern',
+            schema => ['array*', of=>'re*'],
+            tags => ['category:filtering'],
+        },
+        exclude_table_patterns => {
+            'x.name.is_plural' => 1,
+            'x.name.singular' => 'exclude_table_pattern',
+            schema => ['array*', of=>'re*'],
+            tags => ['category:filtering'],
+        },
         stop_after_table => {
             schema => 'str*',
         },
@@ -354,14 +374,37 @@ $SPEC{mysql_split_sql_dump_per_table} = {
         # XXX overwrite
     },
 };
-sub mysql_split_sql_dump_per_table {
+sub mysql_sql_dump_extract_tables {
     my %args = @_;
 
+    my $stop_after_tbl  = $args{stop_after_table};
+    my $stop_after_tpat = $args{stop_after_table_pattern};
+    my $inc_tbl  = $args{include_tables};
+    $inc_tbl  = undef unless $inc_tbl  && @$inc_tbl;
+    my $inc_tpat = $args{include_table_patterns};
+    $inc_tpat = undef unless $inc_tpat && @$inc_tpat;
+    my $exc_tbl  = $args{exclude_tables};
+    $exc_tbl  = undef unless $exc_tbl  && @$exc_tbl;
+    my $exc_tpat = $args{exclude_table_patterns};
+    $exc_tpat = undef unless $exc_tpat && @$exc_tpat;
+    my $has_tbl_filters = $inc_tbl || $inc_tpat || $exc_tbl || $exc_tpat;
+
     my ($prevtbl, $curtbl, $pertblfile, $pertblfh);
 
+    my $code_tbl_is_included = sub {
+        my $tbl = shift;
+        return 0 if $exc_tbl  && (grep { $tbl eq $_ } @$exc_tbl );
+        return 0 if $exc_tpat && (grep { $tbl =~ $_ } @$exc_tpat);
+        return 1 if $inc_tbl  && (grep { $tbl eq $_ } @$inc_tbl );
+        return 1 if $inc_tpat && (grep { $tbl =~ $_ } @$inc_tpat);
+        if ($inc_tbl || $inc_tpat) { return 0 } else { return 1 }
+    };
+
     # we use direct <>, instead of cmdline_src for speed
+    my %seentables;
     while (<>) {
-        if (/^(?:CREATE TABLE) `(.+)`/) {
+        if (/^(?:-- Table structure for table|-- Dumping data for table|CREATE TABLE IF NOT EXISTS|CREATE TABLE|DROP TABLE IF EXISTS) `(.+)`/) {
+            goto L1 if $seentables{$1}++;
             $prevtbl = $curtbl;
             if (defined $prevtbl && $args{stop_after_table} && $prevtbl eq $args{stop_after_table}) {
                 last;
@@ -370,14 +413,16 @@ sub mysql_split_sql_dump_per_table {
             }
             $curtbl = $1;
             $pertblfile = "$curtbl";
-            if (defined $prevtbl) {
-                close $pertblfh;
-                #say "Finished writing $pertblfile";
+            if ($has_tbl_filters && !$code_tbl_is_included->($curtbl)) {
+                warn "SKIPPING table $curtbl\n";
+                undef $pertblfh;
+            } else {
+                warn "Writing $pertblfile ...\n";
+                open $pertblfh, ">", $pertblfile or die "Can't open $pertblfile: $!";
             }
-            warn "Writing $pertblfile ...\n";
-            open $pertblfh, ">", $pertblfile or die "Can't open $pertblfile: $!";
         }
-        next unless $curtbl;
+      L1:
+        next unless $curtbl && $pertblfh;
         print $pertblfh $_;
     }
 
@@ -397,7 +442,22 @@ $SPEC{mysql_run_sql_files} = {
         },
         %args_database,
         # XXX output_file_pattern
-        # XXX overwrite
+        overwrite_when => {
+            summary => 'Specify when to overwrite existing .txt file',
+            schema => ['str*', in=>[qw/none older always/]],
+            default => 'none',
+            description => <<'_',
+
+`none` means to never overwrite existing .txt file. `older` overwrites existing
+.txt file if it's older than the corresponding .sql file. `always` means to
+always overwrite existing .txt file.
+
+_
+            cmdline_aliases => {
+                o         => {summary=>'Shortcut for --overwrite_when=older' , is_flag=>1, code=>sub {$_[0]{overwrite_when} = 'older' }},
+                O         => {summary=>'Shortcut for --overwrite_when=always', is_flag=>1, code=>sub {$_[0]{overwrite_when} = 'always'}},
+            },
+        },
     },
     deps => {
         prog => 'mysql',
@@ -406,10 +466,30 @@ $SPEC{mysql_run_sql_files} = {
 sub mysql_run_sql_files {
     my %args = @_;
 
+    my $ov_when = $args{overwrite_when} // 'none';
+
     for my $sqlfile (@{ $args{sql_files} }) {
+
         my $txtfile = $sqlfile;
         $txtfile =~ s/\.sql$/.txt/i;
         if ($sqlfile eq $txtfile) { $txtfile .= ".txt" }
+
+        if (-f $txtfile) {
+            if ($ov_when eq 'always') {
+                $log->debugf("Overwriting existing %s ...", $txtfile);
+            } elsif ($ov_when eq 'older') {
+                if ((-M $txtfile) > (-M $sqlfile)) {
+                    $log->debugf("Overwriting existing %s because it is older than the corresponding %s ...", $txtfile, $sqlfile);
+                } else {
+                    $log->infof("%s already exists and newer than corresponding %s, skipped", $txtfile, $sqlfile);
+                    next;
+                }
+            } else {
+                $log->infof("%s already exists, we never overwrite existing .txt file, skipped", $txtfile);
+                next;
+            }
+        }
+
         $log->infof("Running SQL file '%s' and putting result to '%s' ...",
                     $sqlfile, $txtfile);
         my $cmd = join(
@@ -440,7 +520,7 @@ App::MysqlUtils - CLI utilities related to MySQL
 
 =head1 VERSION
 
-This document describes version 0.005 of App::MysqlUtils (from Perl distribution App-MysqlUtils), released on 2017-06-15.
+This document describes version 0.006 of App::MysqlUtils (from Perl distribution App-MysqlUtils), released on 2017-06-15.
 
 =head1 SYNOPSIS
 
@@ -669,6 +749,14 @@ Arguments ('*' denotes required arguments):
 
 =item * B<database>* => I<str>
 
+=item * B<overwrite_when> => I<str> (default: "none")
+
+Specify when to overwrite existing .txt file.
+
+C<none> means to never overwrite existing .txt file. C<older> overwrites existing
+.txt file if it's older than the corresponding .sql file. C<always> means to
+always overwrite existing .txt file.
+
 =item * B<sql_files>* => I<array[filename]>
 
 =back
@@ -685,11 +773,11 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 mysql_split_sql_dump_per_table
+=head2 mysql_sql_dump_extract_tables
 
 Usage:
 
- mysql_split_sql_dump_per_table(%args) -> [status, msg, result, meta]
+ mysql_sql_dump_extract_tables(%args) -> [status, msg, result, meta]
 
 Parse SQL dump and spit out tables to separate files.
 
@@ -699,6 +787,14 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<exclude_table_patterns> => I<array[re]>
+
+=item * B<exclude_tables> => I<array[str]>
+
+=item * B<include_table_patterns> => I<array[re]>
+
+=item * B<include_tables> => I<array[str]>
+
 =item * B<stop_after_table> => I<str>
 
 =item * B<stop_after_table_pattern> => I<re>
